local o = mOnWardrobe

---------------------------------------------------------------
--  Default Settings (mOnWDSave)
---------------------------------------------------------------

o.defaultSettings = {}
o.defaultSettings.hideList = false

---------------------------------------------------------------
--  Main Initialization
---------------------------------------------------------------

o.instances = {}
o.version = GetAddOnMetadata("mOnArs_WardrobeHelper", "Version");

local TYPES = {
	'Boss drop', 'Quests', 'Vendor', 'World/Instance Drop', 'Achievements', 'Professions'
}

TYPES[0] = 'Other'

o.TYPES = TYPES
o.SPECIAL = {}

for i = 0,#TYPES do
	o.SPECIAL[TYPES[i]] = true
end

o.menuItems = {}

---------------------------------------------------------------
--  Debug Methods
---------------------------------------------------------------

o.debug = {}
o.debug.printInstances = function()
	local text = ""
	for k, v in orderedPairs(o.instances) do
		text = text .. "~" .. k
	end
	print(text)
end

---------------------------------------------------------------
--  Methods
---------------------------------------------------------------

local function isCollected(sources)
	for i = 1,#sources do
		if sources[i].isCollected then
			return true
		end
	end
	return false
end

o.updateMissingVisuals = function()
	o.instances = {}
	for i = 1,30 do
		local appearances = C_TransmogCollection.GetCategoryAppearances(i)
		if appearances then
			for j = 1,#appearances do
				if appearances[j].isCollected == false then
					local sources = C_TransmogCollection.GetAppearanceSources(appearances[j].visualID)
					if sources and isCollected(sources) == false then
						for m = 1,#sources do
							local i1, i2, b1, i3, b2, itemString, visualString, sourceType = C_TransmogCollection.GetAppearanceSourceInfo(sources[m].sourceID)

							-- 1 = Boss Raid Drops
							if sourceType == 1 then
								local drops = C_TransmogCollection.GetAppearanceSourceDrops(sources[m].sourceID)
								for k=1,#drops do
									local inst = drops[k]
									if o.instances[inst.instance] == nil then
										o.instances[inst.instance] = {}
									end
									if #inst.difficulties == 0 then
										inst.difficulties[1] = "Normal"
									end
									for l=1,#inst.difficulties do
										local dif = inst.difficulties[l]
										if o.instances[inst.instance][dif] == nil then
											o.instances[inst.instance][dif] = {}
										end
										if o.instances[inst.instance][dif][inst.encounter] == nil then
											o.instances[inst.instance][dif][inst.encounter] = {}
										end
										local item = {}
										item.link = itemString
										item.id = string.match(itemString, 'item:([^:]*):')
										item.visualID = appearances[j].visualID
										item.sourceID = sources[m].sourceID
										table.insert(o.instances[inst.instance][dif][inst.encounter], item)
									end
								end
							else
								local type = TYPES[sourceType]
								if o.instances[type] == nil then
									o.instances[type] = {}
									o.instances[type]['Normal'] = {}
								end
								if o.instances[type]['Normal'][i] == nil then
									o.instances[type]['Normal'][i] = {}
								end
								local item = {}
								item.link = itemString
								item.id = string.match(itemString, 'item:([^:]*):')
								item.visualID = appearances[j].visualID
								item.sourceID = sources[m].sourceID
								table.insert(o.instances[type]['Normal'][i], item)
							end
						end
					end
				end
			end
		end
	end

	local exps = {}
	for i = 1,#o.EXPS do
		exps[o.EXPS[i]] = {}
		for j = 1,#o.PARTY do
			exps[o.EXPS[i]][o.PARTY[j]] = {}
		end
	end

	local expsBonus = {}
	expsBonus['Unknown'] = {}
	expsBonus['Special'] = {}

	for k, v in orderedPairs(o.instances) do
		if o.categorization[k] then
			local exp = o.EXPS[o.categorization[k][1]]
			local par = o.PARTY[o.categorization[k][2]]
			table.insert(exps[exp][par], k)
		elseif o.SPECIAL[k] then
			table.insert(expsBonus['Special'], k)
		else
			table.insert(expsBonus['Unknown'], k)
		end
	end

	o.menuItems = {}

	for j = 1,#o.EXPS do
		local tmp = {}
		tmp.name = o.EXPS[j]
		tmp.category = true
		table.insert(o.menuItems, tmp)
		for k1, v1 in pairs(exps[o.EXPS[j]]) do
			for i = 1,#v1 do
				local tmp = {}
				tmp.name = v1[i]
				tmp.type = k1
				table.insert(o.menuItems, tmp)
			end
		end
	end

	for k,v in pairs(expsBonus) do
		local tmp = {}
		tmp.name = k
		tmp.category = true
		table.insert(o.menuItems, tmp)
		for i = 1,#v do
			local tmp = {}
			tmp.name = v[i]
			table.insert(o.menuItems, tmp)
		end
	end

	o.GUImainPage(1)
end

o.refreshInstance = function(instance)
	if o.instances[instance] == nil then
		return
	end
	for difficulty, bosses in pairs(o.instances[instance]) do
		for boss, items in pairs(bosses) do
			for i=#items,1,-1 do
				local sources = C_TransmogCollection.GetAppearanceSources(items[i].visualID)
				if isCollected(sources) then
					table.remove(items, i)
				end
			end
		end
	end
end

o.GUIopen = function()
	o.GUImainPage(o.CURRENT_PAGE)
	mOnWD_MainFrame.hideList:SetChecked(mOnWDSave.hideList)
	mOnWD_MainFrame:Show()
	o.GUIcurrentInstance()
end

o.fixSettings = function()
	if mOnWDSave == nil then mOnWDSave = {} end
	for k, v in pairs(o.defaultSettings) do
		if mOnWDSave[k] == nil then mOnWDSave[k] = v end
	end
end

---------------------------------------------------------------
--  Slash Commands
---------------------------------------------------------------

SLASH_MONWARDROBE1 = '/mwd';

local function handler(msg, editbox)
	o.GUIopen()
end

SlashCmdList["MONWARDROBE"] = handler;

---------------------------------------------------------------
--  Events
---------------------------------------------------------------

o.eventFrame = CreateFrame("Frame")
local f = o.eventFrame
f:RegisterEvent("ADDON_LOADED")

function f:OnEvent(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18, arg19, arg20, arg21, arg22, arg23)
	if (event == "ADDON_LOADED") and (arg1 == "mOnArs_WardrobeHelper") then
		o.fixSettings()
	end
end

function f:OnUpdate(arg1)
	if WardrobeCollectionFrame then
		if WardrobeCollectionFrame.NavigationFrame then
			if mOnWD_MainFrame_bShow == nil then
				local b = CreateFrame("BUTTON","mOnWD_MainFrame_bShow",WardrobeCollectionFrame.NavigationFrame,"UIPanelButtonTemplate");
				b:SetPoint("BOTTOMLEFT",25,25);
				b:SetText("Wardrobe Helper")
				b:SetHeight(25);
				b:SetWidth(120);
				b:SetScript("OnClick", function()
						o.GUIopen()
					end)
			end
		end
	end
end

f:SetScript("OnUpdate", f.OnUpdate)
f:SetScript("OnEvent", f.OnEvent)
